(**

  This module defines a class which creates an Open Tools API wizard for the
  Borland / CodeGear IDEs.

  @Author  David Hoyle
  @Date    18 Sep 2009
  @Version 2.0

**)
Unit BlogOTAExampleWizard;

Interface

Uses
  ToolsAPI, ExtCtrls, Menus; // Added ExtCtrls & Menus

Type
  (** A class to define the OTA IDE wizard. **)
  TBlogOTAExampleWizard = Class(TInterfacedObject, IOTAWizard)
  Private
    FTimer       : TTimer;
    FCounter     : Integer;
    FAutoSaveInt : Integer;
    FPrompt      : Boolean;
    FMenuItem    : TMenuItem;
    FINIFileName : String;
    FSucceeded: Boolean;
    Procedure SaveModifiedFiles;
    Procedure LoadSettings;
    Procedure SaveSettings;
    Procedure InstallMenu;
    Procedure TimerEvent(Sender : TObject);
    Procedure MenuClick(Sender : TObject);
  Protected
    procedure Execute;
    function  GetIDString: string;
    function  GetName: string;
    function  GetState: TWizardState;
    procedure AfterSave;
    procedure BeforeSave;
    procedure Destroyed;
    procedure Modified;
  Public
    Constructor Create;
    Destructor Destroy; Override;
  End;

  Procedure Register;
  Function InitWizard(Const BorlandIDEServices : IBorlandIDEServices;
    RegisterProc : TWizardRegisterProc;
    var Terminate: TWizardTerminateProc) : Boolean; StdCall;

Exports
  InitWizard Name WizardEntryPoint;

Implementation

Uses
   // Delete Dialogs and add Windows, SysUtils and OptionsForm
  Forms, Windows, SysUtils, OptionsForm, IniFiles;

Var
  (** A private variable to hold the index returned by AddWizard which is needed
      by RemoveWizard. **)
  iWizardIndex : Integer = 0; // Renamed

(**

  This is a common intialisation routine for both package and DLL wizards.

  @precon  BIDES must be a valid instance of the IBorlandIDEServices interface.
  @postcon Returns the instance of the IDE wizard to be registered by the
           startup code.

  @param   BIDES as an IBorlandIDEServices
  @return  a TBlogOTAExampleWizard

**)
Function InitialiseWizard(BIDES : IBorlandIDEServices) : TBlogOTAExampleWizard;

Begin
  Result := TBlogOTAExampleWizard.Create;
  Application.Handle := (BIDES As IOTAServices).GetParentHandle;
End;

(**

  This function is required by DLL wizards so that IDE can load and initialise
  the wizard.

  @precon  None.
  @postcon Called by the IDE to Initialise the Wizard.

  @param   BorlandIDEServices as an IBorlandIDEServices as a constant
  @param   RegisterProc       as a TWizardRegisterProc
  @param   Terminate          as a TWizardTerminateProc as a reference
  @return  a Boolean

**)
Function InitWizard(Const BorlandIDEServices : IBorlandIDEServices;
  RegisterProc : TWizardRegisterProc;
  var Terminate: TWizardTerminateProc) : Boolean; StdCall;

Begin
  Result := BorlandIDEServices <> Nil;
  RegisterProc(InitialiseWizard(BorlandIDEServices));
End;

(**

  This procedure is required for Package wizard in order for the IDE to load
  and initialise the wizard.

  @precon  None.
  @postcon Called by the IDE to Initialise the Wizard.

**)
Procedure Register;

Begin
  iWizardIndex := (BorlandIDEServices As IOTAWizardServices).AddWizard(
    InitialiseWizard(BorlandIDEServices));
End;

{ TBlogOTAExampleWizard }

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Not called for IOTAWizard interface.

**)
procedure TBlogOTAExampleWizard.AfterSave;
begin
  // Do nothing
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Not called for IOTAWizard interface.

**)
procedure TBlogOTAExampleWizard.BeforeSave;
begin
  // Do nothing
end;

(**

  This is a constructor for the TBlogOTAWizardWizard class.

  @precon  None.
  @postcon Intialises the wizard and creates a menu for settings the options.

**)
constructor TBlogOTAExampleWizard.Create;

var
  iSize : DWORD;

begin
  FMenuItem := Nil;
  FCounter := 0;
  FAutoSaveInt := 300; // Default 300 seconds (5 minutes)
  FPrompt := True;     // Default to True
  // Create INI file same as add module + '.INI'
  SetLength(FINIFileName, MAX_PATH);
  iSize := MAX_PATH;
  iSize := GetModuleFileName(hInstance, PChar(FINIFileName), iSize);
  SetLength(FINIFileName, iSize);
  FINIFileName := ChangeFileExt(FINIFileName, '.INI');
  LoadSettings;
  FSucceeded := False;
  FTimer := TTimer.Create(Nil);
  FTimer.Interval := 1000; // 1 second
  FTimer.OnTimer := TimerEvent;
  FTimer.Enabled := True;
end;

(**

  This is a destructor for the TBlogOTAExampleWizard class.

  @precon  None.
  @postcon Frees memory used by the wizard.

**)
destructor TBlogOTAExampleWizard.Destroy;
begin
  SaveSettings;
  FMenuItem.Free;
  FTimer.Free;
  Inherited Destroy;
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Since we have a destructor for this class we do not need to use this
           method.

**)
procedure TBlogOTAExampleWizard.Destroyed;
begin
  // Do nothing
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Not called for IOTAWizard interface.

**)
procedure TBlogOTAExampleWizard.Execute;
begin
  // Do nothing
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Returns a unique identifier string for the wizard.

  @return  a string

**)
function TBlogOTAExampleWizard.GetIDString: string;
begin
  Result := 'David Hoyle.BlogIOTAExample';
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Returns a name for the wizard.

  @return  a string

**)
function TBlogOTAExampleWizard.GetName: string;
begin
  Result := 'Open Tools API Example';
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Returns an enabled state for the wizard.

  @return  a TWizardState

**)
function TBlogOTAExampleWizard.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

(**

  This method installs the menu in the IDE and registers that it has succeeded.

  @precon  None.
  @postcon Installs the menu in the IDE and registers that it has succeeded.

**)
procedure TBlogOTAExampleWizard.InstallMenu;

Var
  NTAS: INTAServices;
  mmiViewMenu: TMenuItem;
  mmiWindowList: TMenuItem;

begin
  NTAS := (BorlandIDEServices As INTAServices);
  If (NTAS <> Nil) And (NTAS.MainMenu <> Nil) Then
    Begin
      mmiViewMenu := NTAS.MainMenu.Items.Find('View');
      If mmiViewMenu <> Nil Then
        Begin
          mmiWindowList := mmiViewMenu.Find('Window List...');
          If mmiWindowList <> Nil Then
            Begin
              FMenuItem := TMenuItem.Create(mmiViewMenu);
              FMenuItem.Caption := '&Auto Save Options...';
              FMenuItem.OnClick := MenuClick;
              FMenuItem.ShortCut := TextToShortCut('Ctrl+Shift+Alt+A');
              mmiViewMenu.Insert(mmiWindowList.MenuIndex + 1, FMenuItem);
              FSucceeded := True;
            End;
        End;
    End;
end;

(**

  This method loads the wizards settings from the INI file.

  @precon  None.
  @postcon Loads the wizards settings from the INI file.

**)
procedure TBlogOTAExampleWizard.LoadSettings;
begin
  With TIniFile.Create(FINIFileName) Do
    Try
      FAutoSaveInt := ReadInteger('Setup', 'AutoSaveInt', FAutoSaveInt);
      FPrompt := ReadBool('Setup', 'Prompt', FPrompt);
    Finally
      Free;
    End;
end;

(**

  This method is an on click event handler for the Auto Save Options menu item.

  @precon  None.
  @postcon Invokes the options dialogue to allow the user to change the
           interval and prompt settings.

  @param   Sender as a TObject

**)
procedure TBlogOTAExampleWizard.MenuClick(Sender: TObject);
begin
  If TfrmOptions.Execute(FAutoSaveInt, FPrompt) Then
    SaveSettings; // Not really required as is called in destructor.
end;

(**

  This method requires implementing for the IOTAWizard interface.

  @precon  None.
  @postcon Not called for IOTAWizard interface.

**)
procedure TBlogOTAExampleWizard.Modified;
begin
  // Do nothing
end;

(**

  This method iterates over the open files in the editor buffer and saves them
  IF the file is modified.

  @precon  None.
  @postcon Iterates over the open files in the editor buffer and saves them
           IF the file is modified.

**)
procedure TBlogOTAExampleWizard.SaveModifiedFiles;

Var
  Iterator : IOTAEditBufferIterator;
  i : Integer;

begin
  If (BorlandIDEServices As IOTAEditorServices).GetEditBufferIterator(Iterator) Then
    Begin
      For i := 0 To Iterator.Count - 1 Do
        If Iterator.EditBuffers[i].IsModified Then
          Iterator.EditBuffers[i].Module.Save(False, Not FPrompt);
    End;
end;

(**

  This method saves the wizards settings to the INI file.

  @precon  None.
  @postcon Saves the wizards settings to the INI file.

**)
procedure TBlogOTAExampleWizard.SaveSettings;

begin
  With TIniFile.Create(FINIFileName) Do
    Try
      WriteInteger('Setup', 'AutoSaveInt', FAutoSaveInt);
      WriteBool('Setup', 'Prompt', FPrompt);
    Finally
      Free;
    End;
end;

(**

  This is an on timer event handler for the timer control.

  @precon  None.
  @postcon Increments the counter and saves the modified files IF counter is
           greater than the auto save interval.

  @param   Sender as a TObject

**)
procedure TBlogOTAExampleWizard.TimerEvent(Sender: TObject);
begin
  Inc(FCounter);
  If FCounter >= FAutoSaveInt Then
    Begin
      FCounter := 0;
      SaveModifiedFiles;
    End;
  If Not FSucceeded Then
    InstallMenu;
end;

(** Do nothing **)
Initialization
(** Removes the wizard using the Wizard services IF iWiardIndex > 0. **)
Finalization
  If iWizardIndex > 0 Then
    (BorlandIDEServices As IOTAWizardServices).RemoveWizard(iWizardIndex);
End.
